/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include "F28x_Project.h"     // Device Headerfile and Examples Include File
#include "IQmathLib.h"

/////
// Defines
////
////// Below sets the PWM Frequency to ~2MHz /////////
#define EPWM12_TIMER_TBPRD  25  // Period register
#define EPWM12_MAX_CMPA     20
#define EPWM12_MIN_CMPA     10
#define EPWM12_MAX_CMPB     20
#define EPWM12_MIN_CMPB     10

#define EPWM_CMP_UP           1
#define EPWM_CMP_DOWN         0

// Definitions are for the FCDCBC Register per LED Driver ///////
#define Function                  0x00F     /////// Value for Function Control bits, this sets auto repeat on and dc ranges to high
//#define BCbits                    0x808080
#define DCvalue                   0xFFF     /////// Set to Max  Value

// Definitions for the OPT3001 Light Level Sensor
// The only registers used in this example are the Result and Configuration Registers
Uint16 OPT3001 = 0x44;
Uint16 Result = 0x00;
Uint16 Configuration = 0x01;
#define Divisor 750.0

//// For Delay Function, Short, Medium and long delay

#define quick   500000
#define midpace 5000000
#define slow    7000000
//
// Globals
//
typedef struct
{
    volatile struct EPWM_REGS *EPwmRegHandle;
    Uint16 EPwm_CMPA_Direction;
    Uint16 EPwm_CMPB_Direction;
    Uint16 EPwmTimerIntCount;
    Uint16 EPwmMaxCMPA;
    Uint16 EPwmMinCMPA;
    Uint16 EPwmMaxCMPB;
    Uint16 EPwmMinCMPB;

}EPWM_INFO;

///IQ4 variables for tempsense calculation////
/**/
_iq4 sensorSample;
_iq4 Res;
_iq4 scale;
_iq4 tcheck;
/**/

unsigned short sdata;  // send data
unsigned short rdata;  // send data

//////// These Variables are used for the ambient light sensor opt3001,
///// As the result bits require some manipulation to first find the lux range and then calculate the actual reading within this range//////
///// The brightness, GSscale is and scalar is to set the GS bits equal to the brightness being read so a gradient can be observed    //////

unsigned char exponent;
unsigned int mantissa;
unsigned int brightness = 0;
unsigned long GSScale;
float Scalar;

////// Thes Variables are for if you want to observe affect of changing BC manual or mapping to output of ALS,
//Outside of the Animation loop

unsigned long setting; // setting is set equal to BC bits within swetting BC function //
unsigned long BCbits; //BC bits are teh actual BC bits to be written

////// Grayscale array for GS write ///////

unsigned short arrayGS[7][24] = {{0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                         {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                         {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                         {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                         {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                         {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                         {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};

/////// Array of Function Control, Brightness Control and Dot Correction

unsigned short arrayFCBCDC[7][24] = {{0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                             {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                             {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                             {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                             {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                             {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                             {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};

//9 different Image arrays, the 12 bit values can be varied to vary brightness per LED.

unsigned short Ball[8][18] =  {{0x000, 0x020, 0x150, 0x020, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x150, 0xAAA, 0x150, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x020, 0x150, 0x020, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};

unsigned short Ball2[8][18] = {{0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0xAAA, 0xAAA, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0xAAA, 0xAAA, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};

unsigned short Line[8][18] =  {{0x000, 0x000, 0x010, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x050, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x150, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x444, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0xAAA, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x888, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x200, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x050, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};

unsigned short Lights[8][18] = {{0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF},
                        {0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF, 0xFFF}};

unsigned short ti[8][18] = {{0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x350, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x350, 0x350, 0x000, 0x350},
                            {0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x350, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x000, 0x000, 0x350},
                            {0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x000, 0x000, 0x350, 0x000, 0x000, 0x350},
                            {0x000, 0x000, 0x000, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000},
                            {0x000, 0x000, 0x000, 0x000, 0x350, 0x000, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000},
                            {0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x000, 0x350, 0x350, 0x350, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                            {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                            {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x350, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};


//// Array For testing brightness of two LEDs with test points and resistors connected ///////

unsigned short LEDs[8][18] =  {{0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x800, 0x800},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000},
                       {0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000, 0x000}};

//Lookuptable           // Order of LED Drivers
                        // Needs to be edited to correspond to order of spi Routing between LED Drivers

unsigned short LutA[8][18] = {{0, 0, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4},
                      {0, 0, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4},
                      {0, 0, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4},
                      {0, 0, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4},
                      {0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3},
                      {0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3},
                      {0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3},
                      {0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3}};


unsigned short LutB[8][18] = {{3, 18, 4, 0, 2, 23, 21, 19, 4, 0, 2, 23, 21, 19, 5, 0, 21, 20},
                      {5, 20, 3, 5, 1, 22, 20, 18, 3, 5, 1, 22, 20, 18, 3, 2, 23, 18},
                      {0, 21, 7, 8, 9, 12, 17, 16, 7, 8, 9, 12, 17, 16, 8, 9, 12, 17},
                      {2, 23, 6, 10,11,14, 13, 15, 6, 10,11,14, 13, 15, 6, 11,14, 15},
                      {9, 12, 4, 0, 2, 22, 21, 19, 4, 0, 2, 23, 21, 19, 5, 0, 23, 21},
                      {11,14, 3, 5, 1, 23, 20, 18, 3, 5, 1, 22, 20, 18, 3, 2, 20, 18},
                      {6, 15, 7, 8, 9, 12, 17, 16, 7, 8, 9, 12, 17, 16, 8, 11,12, 17},
                      {8, 17, 6, 10,11,14, 13, 15, 6, 10,11,14, 13, 15, 6, 9, 14, 15}};

EPWM_INFO epwm12_info;

/* Delay Loop */
void delay(long milli);

/* Initialise spi */
void spi_xmit(Uint16 a);
void spi_fifo_init(void);
void spi_init(void);

////// Led Driver Functions Follow/////////
///// Animation allows specified image to move in the x or y direction by specified number of frames /////////

void animation(unsigned short imageArray[8][18],int frames,int yMove, int xMove, long delay);

////// This function creates the grayscale array to be written in the GS Write //////////
void createGS(unsigned short array[8][18],int frameCount, int xx, int yy);

////// This Function sets the FC, BC and DC bits for eaxh LED driver ////////
void FCBCDC(int FC, unsigned long BC, int DC);

////// This Function sends teh created GS or FCBCDC array over spi to the LED Drivers/////
void dataSend(unsigned short sarray[7][24]);

// Initialise ADC ///

void hal_InitADC(void);

///// Get Temperature in degrees C from LMT87  ////////
void tempC(void);

///// Initiation I2C for ALS /////////

void init_i2cALS();

///// Variables to be uses with the Ambient Light sensor
unsigned long lux;
unsigned char data[2];

//// These bytes are used for when writing to configuration register of the OPT3001
unsigned char byte1 = 0xC2;
unsigned char byte2 = 0x10;

//// opt3001 requires to set up the amount of bits to be written, and then Write to to specified register.
void I2C_setup_write();
void I2C_write();
//// Opt3001 requires a write to the register to be read from in this case pointing to the result register and then reading from it.
void I2C_setup_Read();
void I2C_setup_Read();
Uint16 I2C_Read(unsigned char* data);

//// Value returned from the opt3001 Ambient light sensor, can be converted straight to a brightness/lux value
unsigned int convert_lux_to_brightness(unsigned int lux_to_brightness);
unsigned long convert_to_lux(unsigned int exponent, unsigned int mantissa);

void setting_BC(unsigned long setting);

/* Error check function */

void error(void);

/* Initialise EPWM12 */

void InitEPwm12Example(void);

__interrupt void epwm12_isr(void);
void update_compare(EPWM_INFO*);

void main(void)

{
    //
    // Initialize System Control:
    // PLL, WatchDog, enable Peripheral Clocks
    // This example function is found in the F2837xD_SysCtrl.c file.

    InitSysCtrl();
       EALLOW;
       ClkCfgRegs.PERCLKDIVSEL.bit.EPWMCLKDIV = 1;
       EDIS;

    // Initialise Spi GPIO:

       InitSpiaGpio();
       InitSpibGpio();

    // Initialise EPWM GPIO'S'
       CpuSysRegs.PCLKCR2.bit.EPWM12=1;
       InitEPwm12Gpio();


    //   Init_I2C();
    //////  Initialize I2CA clock ///////
       EALLOW;
       CpuSysRegs.PCLKCR9.bit.I2C_A = 1;
       EDIS;

    //////// Initialize I2CA GPIO'S) that will be used for ALS//////
       I2cAGpioConfig(I2C_A_GPIO63104_GPIO105);

    // Clear all __interrupts and initialize PIE vector table:
    // Disable CPU __interrupts
       DINT;

    // Initialize PIE control registers to their default state.
       InitPieCtrl();

    // Disable CPU __interrupts and clear all CPU __interrupt flags:
       IER = 0x0000;
       IFR = 0x0000;

    // Initialize the PIE vector
       InitPieVectTable();

       EALLOW; // This is needed to write to EALLOW protected registers
       PieVectTable.EPWM12_INT = &epwm12_isr;
       EDIS;   // This is needed to disable write to EALLOW protected registers

    // initialize the ePWM
       EALLOW;
       CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 0;
       EDIS;

       InitEPwm12Example();

       EALLOW;
       CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;
       EDIS;

    // Initialize the Spi Peripheral:
       spi_fifo_init();   // Initialize the Spi FIFO
       spi_init();        // init SPI

       /* init the ADC */
       hal_InitADC();

       // Enable CPU INT3 which is connected to EPWM1-3 INT:
       //
       IER |= M_INT3;

       // Enable EPWM INTn in the PIE: Group 3 interrupt 1-3
       //
       PieCtrlRegs.PIEIER3.bit.INTx12 = 1;

       // Enable global Interrupts and higher priority real-time debug events:
       //
       EINT;  // Enable Global interrupt INTM
       ERTM;  // Enable Global realtime interrupt DBGM

       //////// Animation Section, this section of the animation, Move different specified images across the screen //////////
       //////// The Images that are passed into this function are the Global [8][18] arrays which specify which     //////////
       //////// LED's of the 144 will be on to create the image such as a ball in the first instance                //////////
       //////// The speed at which the image moves across the screen is set by the Quick, midpace and slow          //////////
       //////// As the 5th Variable in the animation function                                                       //////////

       while(1){
       animation(Ball,18,1,0,quick);
       animation(Line,18,1,0,quick);
       animation(Line,25,1,1,quick);
       animation(ti,1,0,0,15000000);
       }
}

// Step 7. Insert all local Interrupt Service Routines (ISRs) and functions here:

void delay(long milli)
{
    long      i;
    for (i = 0; i < milli; i++) {}
}

////Move to before EPWM ////// CA

void spi_init()
{
    SpiaRegs.SPICCR.all =0x000B;                 // Reset on, rising edge, 16-bit char bits
    SpiaRegs.SPICTL.all =0x000E;                 // Enable master mode, normal phase,
                                                 // enable talk, and SPI int disabled.
    SpiaRegs.SPIBRR.all =0x007F;
    SpiaRegs.SPICCR.all =0x008B;                 // Relinquish SPI from Reset
    SpiaRegs.SPIPRI.bit.FREE = 1;                // Set so breakpoints don't disturb xmission

    SpibRegs.SPICCR.all =0x000B;                 // Reset on, rising edge, 16-bit char bits
    SpibRegs.SPICTL.all =0x000E;                 // Enable master mode, normal phase,
                                                 // enable talk, and SPI int disabled.
    SpibRegs.SPIBRR.all =0x007F;
    SpibRegs.SPICCR.all =0x008B;                 // Relinquish SPI from Reset
    SpibRegs.SPIPRI.bit.FREE = 1;                // Set so breakpoints don't disturb xmission
}

void spi_xmit(Uint16 a)
{
    SpiaRegs.SPITXBUF=a;
    SpibRegs.SPITXBUF=a;
}

void spi_fifo_init()
{
// Initialize SPI FIFO registers
    SpiaRegs.SPIFFTX.all=0xE040;
    SpiaRegs.SPIFFRX.all=0x2044;
    SpiaRegs.SPIFFCT.all=0x0;
// Initialize SPI FIFO registers
    SpibRegs.SPIFFTX.all=0xE040;
    SpibRegs.SPIFFRX.all=0x2044;
    SpibRegs.SPIFFCT.all=0x0;
}

void animation(unsigned short imageArray[8][18],int frames,int yMove, int xMove, long milli){
    int f = 0;
    for(f=0; f<frames; f++){

        tempC();                                ///////// Temperature sensor reading in degree C from the LMT87-Q1 //////////
        GSScale = OPT3001_Meas();
        Scalar = GSScale/Divisor;
        GpioDataRegs.GPBTOGGLE.bit.GPIO32 = 1;  //toogle Blank signal
        GpioDataRegs.GPDTOGGLE.bit.GPIO125 = 1; //toogle Latch signal
        createGS(imageArray,f,yMove,xMove);
        dataSend(arrayGS);
        GpioDataRegs.GPDTOGGLE.bit.GPIO125 = 1;
        BCbits = 0x0FF;                         // Set BC bits to what you wish here, they are arranged into registers in next line
        FCBCDC(Function,BCbits,DCvalue);        // Creates FCBCDC array
        dataSend(arrayFCBCDC);
        GpioDataRegs.GPDTOGGLE.bit.GPIO125 = 1;
        GpioDataRegs.GPDTOGGLE.bit.GPIO125 = 1;
        GpioDataRegs.GPBTOGGLE.bit.GPIO32 = 1;
        delay(milli);
    }
}

/////// This Function Creates the GS array, and also allows for scaling to the Ambient Light sensor,
/////// if you don not require scaling remove the * Scalar
void createGS(unsigned short array[8][18],int frameCount, int xx, int yy){

    int j = 0;
    int i = 0;
    for(j=0; j<8; j++){
           for(i=0; i<18; i++){
               arrayGS[LutA[j][i]][LutB[j][i]] = array[abs(j + yy*frameCount)%8][abs(i + xx*frameCount)%18]*Scalar;
           }
       }
}

////// Here the FCBCDC array is created, it organises data input, into separate FC, BC and DC Registers of different sizes
void FCBCDC(int FC, unsigned long BC, int DC){

    int r = 0;
    int m = 0;
    for (r=0; r < 7; r++){
        for (m=0; m < 6; m++){
            arrayFCBCDC[r][m]=0x000;
        }

        arrayFCBCDC[r][6]= FC >> 12;
        arrayFCBCDC[r][7]= FC;
        arrayFCBCDC[r][8]= BC << 4 | BC >> 4;       //BC 24 bits
        arrayFCBCDC[r][9]= BC << 8 & 0x0F00 | BC;

    // 14 remaining 12b registers are DC (24 7b registers)
        int m = 10;
        for(m = 10; m <= 24; m++){

            arrayFCBCDC[r][m] = DC;
        }
}
}

///// Here is where the data is actually sent over SPI to the LED drivers
void dataSend(unsigned short sarray[7][24])
{
    int j = 0;
    int i = 0;
    for(j=0; j<7; j++){
        for(i=0; i<24; i++){

            sdata = sarray[j][i] << 4;
            // Transmit data
            spi_xmit(sdata);
            // Wait until data is received
            while(SpibRegs.SPIFFRX.bit.RXFFST !=1) { }
            // Check against sent data
            rdata = SpibRegs.SPIRXBUF;
       }
    }
}

/// This function takes the reading of the ambient light sensor, Inside this function describes teh proceeding I2C related Functions //////
Uint16 OPT3001_Meas()
{
       init_i2cALS();        /// Initialize I2C for ambient light sensor ALS  ////
       I2C_setup_write();   /////// Set Up Write to config Register //////
       I2C_write();         ////// Write Values to configuration register //////
       I2C_setup_Read();   ////// write to result register to allow to read result from the register
       I2C_Read(data);     ////// Read the result

       //Extract the upper four bits of exponent and the lower 12 bits of mantissa from the raw result value
       //The Result value must be converted into a meaningful value
       exponent = data[0] >> 4;
       mantissa = (data[0] << 8 | data[1]) & 0x0FFF;

       //Take the exponent and mantissa values and calculate the corresponding lux value
       lux = convert_to_lux(exponent, mantissa);

      //Convert the lux measurment into a brightness value and split it into the upper and lower bytes
      //The current_position multiplier can be adjusted as necessary to help highlight the changes in
      //the sensor position variable
      brightness = convert_lux_to_brightness(lux);

      return brightness;
}

void init_i2cALS()
{
       I2caRegs.I2CMDR.bit.IRS = 0;          /// Put I2C Module into reset state


       I2caRegs.I2CPSC.all = 5;              //// Configure Module Clock to 10MHz as has to be 7 - 12

       //Set I2C clk High and low times

       I2caRegs.I2CCLKL = 120;               ///// set to 12.5us
       I2caRegs.I2CCLKH = 120;               /////

       I2caRegs.I2CMDR.bit.IRS = 1;          ///// Take Module out of Reset
}

void I2C_setup_write()
{
    ///////////////Set Up Write //////////////////////

                 // load slave address
                 I2caRegs.I2CSAR.all = (OPT3001 & 0x00FF);

                 // wait for STOP condition
                 while (I2caRegs.I2CMDR.bit.STP != 0);

                 // master mode
                 I2caRegs.I2CMDR.bit.MST = 1;

                 // generate START condition
                 I2caRegs.I2CMDR.bit.STT = 1;


   //              enabling repeat mode,
   //              in which a data byte is transmitted each time the I2CDXR register is written to,
   //              until a STOP or REPEATED START condition is generated, both of which ends repeat mode.

                 // repeat mode
                 I2caRegs.I2CMDR.bit.RM = 1;

   //              We set the TRX bit to switch into transmit mode,
   //              and wait for the XRDY (transmit mode ready)

                 // transmit mode
                 I2caRegs.I2CMDR.bit.TRX = 1;

                 // wait for XRDY flag to transmit data
                 while (I2caRegs.I2CSTR.bit.XRDY != 1);
   ///////////////////////////^^^^ Set Up Write ^^^^^^^^ ////////////////////

}

void I2C_write()
{
    /////////////////////////// Write to set up configuration register ///////////////////////////////
                  /// load the byte we want to send

    //              load data into the transmit register
                  I2caRegs.I2CDXR.all = Configuration;

                  while (I2caRegs.I2CSTR.bit.XRDY != 1 || I2caRegs.I2CSTR.bit.ARDY != 1);

                  // if a NACK is received, clear the NACK bit and stop the transfer
                  if (I2caRegs.I2CSTR.bit.NACK == 1) {
                      I2caRegs.I2CMDR.bit.STP = 1;
                      I2caRegs.I2CSTR.all = I2C_CLR_NACK_BIT;
                      return I2C_NACK_ERROR;
                  }

    //            load data into the transmit register
                  I2caRegs.I2CDXR.all = byte1;

                //// Wait for ACK or NACK //////

                  while (I2caRegs.I2CSTR.bit.XRDY != 1 || I2caRegs.I2CSTR.bit.ARDY != 1);

                // if a NACK is received, clear the NACK bit and stop the transfer
                if (I2caRegs.I2CSTR.bit.NACK == 1) {
                    I2caRegs.I2CMDR.bit.STP = 1;
                    I2caRegs.I2CSTR.all = I2C_CLR_NACK_BIT;
                    return I2C_NACK_ERROR;
                }

    //              load data into the transmit register
                 I2caRegs.I2CDXR.all = byte2;

                  //// Wait for ACK or NACK //////

                 while (I2caRegs.I2CSTR.bit.XRDY != 1 || I2caRegs.I2CSTR.bit.ARDY != 1);

                  // if a NACK is received, clear the NACK bit and stop the transfer
                  if (I2caRegs.I2CSTR.bit.NACK == 1) {
                      I2caRegs.I2CMDR.bit.STP = 1;
                      I2caRegs.I2CSTR.all = I2C_CLR_NACK_BIT;
                      return I2C_NACK_ERROR;
                  }
}

void I2C_setup_Read()
{
    I2C_setup_write();

    /////////////////////////// Write to set up Result register ///////////////////////////////
                 /// load the byte we want to send

    //              load data into the transmit register
               I2caRegs.I2CDXR.all = Result;


                 //// Wait for ACK or NACK //////

               while (I2caRegs.I2CSTR.bit.XRDY != 1 || I2caRegs.I2CSTR.bit.ARDY != 1);

                 // if a NACK is received, clear the NACK bit and stop the transfer
                 if (I2caRegs.I2CSTR.bit.NACK == 1) {
                     I2caRegs.I2CMDR.bit.STP = 1;
                     I2caRegs.I2CSTR.all = I2C_CLR_NACK_BIT;
                     return I2C_NACK_ERROR;
                 }
}

Uint16 I2C_Read(unsigned char* data)
{
    int i = 0;
    ///   Read ///////

    // load slave address
     I2caRegs.I2CSAR.all = (OPT3001 & 0x00FF);

    // wait for STOP condition
     while (I2caRegs.I2CMDR.bit.STP != 0);

     // set data count
     I2caRegs.I2CCNT = 2;

     // receiver mode
     I2caRegs.I2CMDR.bit.TRX = 0;

     // issue repeated start
     I2caRegs.I2CMDR.bit.STT = 1;

     ////// Read Bytes Slave has sent ///////

     for (i = 0; i < 2; ++i) {
         // wait until the data receive register is ready to be read
         while (I2caRegs.I2CSTR.bit.RRDY != 1);

         data[i] = I2caRegs.I2CDRR.all;
     }

return data;
}

///// Function converts the Ambient light sensor reading to lux ////////
unsigned long convert_to_lux(unsigned int exponent, unsigned int mantissa)
{
    // Convert the Result Register into a lux measurement based on formula below:
    // lux = 0.01 * (2^E[3:0]) * R[11:0]

    unsigned int result;
    unsigned int count;
    unsigned long converted_lux;

    result = 1;
    for(count = exponent; count > 0; count--)
    {
        result = result * 2;
    }
    converted_lux = mantissa / 100;
    converted_lux *= result;

    return converted_lux;
}

/*
    This function will take converted lux values and assign an arbitrary brightness
    value based on the lux value
    This function will need to be modified based on the operating environment and conditions
    An external light meter should be used to establish the average brightness level
    This function should be used as a starting point and then tweaked for inidividual applications
*/
unsigned int convert_lux_to_brightness(unsigned int lux)
{
    int i;
    long dx, dy;
    static const unsigned int luxCode[7]=
    {
     60000, 2500, 1000, 500, 250, 100, 0
    };
    static const unsigned int brightLvl[7]=
    {
     4096, 3500, 1500, 500, 100, 50, 10
    };
    static const int count = 7;

    if (lux > luxCode[0]) {
        /* lux is greater than the max element
         * handle error here if you want */
        return brightLvl[0]; /* return minimum element */
    }

    if (lux < luxCode[count-1]) {
        return brightLvl[count-1]; /* return maximum */
    }

    /* find i, such that bitCode[i] <= x < bitCode[i-1] */
    for (i = count-1; i > 0; i--) {
        if (luxCode[i-1] > lux) {
            break;
        }
    }

    /* interpolate */
    dx = luxCode[i-1] - luxCode[i];
    dy = brightLvl[i-1] - brightLvl[i];
    return brightLvl[i] + (lux - luxCode[i]) * dy / dx;
}

/* ADC Init */
void hal_InitADC(void)
{

    Uint16 tempsensor_acqps;

    tempsensor_acqps = 139; //temperature sensor needs at least 700ns
                            //acquisition time
    EALLOW;
    /**/
    //
    //write configurations
    //
    AdcaRegs.ADCCTL2.bit.PRESCALE = 6; //set ADCCLK divider to /4
    AdcSetMode(ADC_ADCA, ADC_RESOLUTION_12BIT, ADC_SIGNALMODE_SINGLE);

    //
    //Set pulse positions to late
    //
    AdcaRegs.ADCCTL1.bit.INTPULSEPOS = 1;

    //
    //power up the ADC
    //
    AdcaRegs.ADCCTL1.bit.ADCPWDNZ = 1;

    /* AIN_A */
    AdcaRegs.ADCSOC0CTL.bit.CHSEL  = 0;    /* ADCINA0 is the input */
    AdcaRegs.ADCSOC0CTL.bit.ACQPS = tempsensor_acqps; //sample window is 100
                                                      //SYSCLK cycles
    AdcaRegs.ADCSOC0CTL.bit.TRIGSEL = 4;    //trigger on GPIO, ADCEXTSOC

    /**/
    AdcaRegs.ADCCTL1.bit.INTPULSEPOS = 1;    /* ADCINT1 trips after AdcResults latch */
    AdcaRegs.ADCINTSEL1N2.bit.INT1E     = 1;    /* Enabled ADCINT1 (PIE ignored) */
    AdcaRegs.ADCINTSEL1N2.bit.INT1CONT  = 1;    /* Enable ADCINT1 Continuous mode */
    /**/
    AdcaRegs.ADCINTSOCSEL1.bit.SOC0 = 1;     /* ADCINT1 will trigger SOC0 */
    AdcaRegs.ADCINTSOCSEL1.bit.SOC1 = 1;
    AdcaRegs.ADCINTSOCSEL1.bit.SOC2 = 1;

    /**/
    AdcaRegs.ADCINTSEL1N2.bit.INT1SEL = 0x02;   /* EOC0 triggers the interrupt */
    /* Initial Trigger of ADC (then triggered automatically by interrupt (EoC of SOC0) */
    AdcaRegs.ADCSOCFRC1.bit.SOC2 = 1;
    EDIS;
}

///// Get Temperature in degrees C from LMT87  ////////
void tempC(void)
{
       //Force start of conversion on SOC0
          AdcaRegs.ADCSOCFRC1.all = 0x01;

        //Wait for end of conversion.
          while(AdcaRegs.ADCINTFLG.bit.ADCINT1 == 0){}  //Wait for ADCINT1
          AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;        //Clear ADCINT1    */

          sensorSample = AdcaResultRegs.ADCRESULT0;

          ////// This Part converts reading into a voltage and then temperature in degrees C ///////
          Res = (((sensorSample*3.0)/4096)*1000)-115;
          scale= -13.567;
          tcheck = (((Res-2365)/scale)+20);
}

///// Put error here so spi can initialise first /////
void error(void)
{
    asm("     ESTOP0");                     // Test failed!! Stop!
    for (;;);
}

// epwm12_isr - EPWM12 ISR
//
__interrupt void epwm12_isr(void)
{
    //
    // Update the CMPA and CMPB values
    //
    update_compare(&epwm12_info);

    //
    // Clear INT flag for this timer
    //
    EPwm12Regs.ETCLR.bit.INT = 1;

    //
    // Acknowledge this interrupt to receive more interrupts from group 3
    //
    PieCtrlRegs.PIEACK.all = PIEACK_GROUP3;
}


//
// InitEPwm12Example - Initialize EPWM12 configuration
//
void InitEPwm12Example()
{
    //
    // Setup TBCLK
    //

    EPwm12Regs.TBPRD = EPWM12_TIMER_TBPRD;         // Set timer period 50 TBCLKs
    EPwm12Regs.TBPHS.bit.TBPHS = 0x0000;          // Phase is 0 offset
    EPwm12Regs.TBCTR = 0x0000;                    // Clear counter


    //
    // Set Compare values
    //
    EPwm12Regs.CMPA.bit.CMPA = EPWM12_MIN_CMPA;    // Set compare A value
    EPwm12Regs.CMPB.bit.CMPB = EPWM12_MIN_CMPB;    // Set Compare B value

    //
    // Setup counter mode
    //
    EPwm12Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN; // Count up and down
    EPwm12Regs.TBCTL.bit.PHSEN = TB_DISABLE;        // Disable phase loading
    EPwm12Regs.TBCTL.bit.HSPCLKDIV = TB_DIV1;       // Clock ratio to SYSCLKOUT
    EPwm12Regs.TBCTL.bit.CLKDIV = TB_DIV1;

    //
    // Setup shadowing
    //
    EPwm12Regs.CMPCTL.bit.SHDWAMODE = CC_SHADOW;
    EPwm12Regs.CMPCTL.bit.SHDWBMODE = CC_SHADOW;
    EPwm12Regs.CMPCTL.bit.LOADAMODE = CC_CTR_ZERO; // Load on Zero
    EPwm12Regs.CMPCTL.bit.LOADBMODE = CC_CTR_ZERO;

    //
    // Set actions
    //
    EPwm12Regs.AQCTLA.bit.CAU = AQ_SET;         // Set PWM2A on event A, up
                                               // count   TBCTR = CMPA = 10 FORCE EPWMA to High
    EPwm12Regs.AQCTLA.bit.CBD = AQ_CLEAR;       // Clear PWM2A on event B when TBCTR=CMPB=10, down
                                               // count force EPWMA to low

    EPwm12Regs.AQCTLB.bit.ZRO = AQ_CLEAR;       // Clear PWM2B on zero, When TBCTR = 0 set EPWMB to low
    EPwm12Regs.AQCTLB.bit.PRD = AQ_SET;         // Set PWM2B on period, when TBCTR = TBPRD Force output high

    //
    // Interrupt where we will change the Compare Values
    //
    EPwm12Regs.ETSEL.bit.INTSEL = ET_CTR_ZERO;    // Select INT on Zero event, enable event TBCTR = 0
    EPwm12Regs.ETSEL.bit.INTEN = 1;               // Enable EPWM INT
    EPwm12Regs.ETPS.bit.INTPRD = ET_3RD;          // Generate INT on 3rd event


    //
    // Information this example uses to keep track
    // of the direction the CMPA/CMPB values are
    // moving, the min and max allowed values and
    // a pointer to the correct ePWM registers
    //
    epwm12_info.EPwm_CMPA_Direction = EPWM_CMP_UP;  // Start by increasing CMPA
    epwm12_info.EPwm_CMPB_Direction = EPWM_CMP_UP;  // & increasing CMPB
    epwm12_info.EPwmTimerIntCount = 0;              // Zero the interrupt counter
    epwm12_info.EPwmRegHandle = &EPwm12Regs;         // Set the pointer to the
                                                   // ePWM module
    epwm12_info.EPwmMaxCMPA = EPWM12_MAX_CMPA;       // Setup min/max CMPA/CMPB
                                                   // values
    epwm12_info.EPwmMinCMPA = EPWM12_MIN_CMPA;
    epwm12_info.EPwmMaxCMPB = EPWM12_MAX_CMPB;
    epwm12_info.EPwmMinCMPB = EPWM12_MIN_CMPB;


}


//
// update_compare - Update the PWM compare values
//
void update_compare(EPWM_INFO *epwm_info)
{
    //
    // Every 10'th interrupt, change the CMPA/CMPB values
    //
    if(epwm_info->EPwmTimerIntCount == 10)
    {
        epwm_info->EPwmTimerIntCount = 0;

        //
        // If we were increasing CMPA, check to see if
        // we reached the max value.  If not, increase CMPA
        // else, change directions and decrease CMPA
        //
        if(epwm_info->EPwm_CMPA_Direction == EPWM_CMP_UP)
        {
            if(epwm_info->EPwmRegHandle->CMPA.bit.CMPA <
               epwm_info->EPwmMaxCMPA)
            {
                epwm_info->EPwmRegHandle->CMPA.bit.CMPA++;
            }
            else
            {
                epwm_info->EPwm_CMPA_Direction = EPWM_CMP_DOWN;
                epwm_info->EPwmRegHandle->CMPA.bit.CMPA--;
            }
        }

        //
        // If we were decreasing CMPA, check to see if
        // we reached the min value.  If not, decrease CMPA
        // else, change directions and increase CMPA
        //
        else
        {
            if(epwm_info->EPwmRegHandle->CMPA.bit.CMPA ==
               epwm_info->EPwmMinCMPA)
            {
                epwm_info->EPwm_CMPA_Direction = EPWM_CMP_UP;
                epwm_info->EPwmRegHandle->CMPA.bit.CMPA++;
            }
            else
            {
                epwm_info->EPwmRegHandle->CMPA.bit.CMPA--;
            }
        }

        //
        // If we were increasing CMPB, check to see if
        // we reached the max value.  If not, increase CMPB
        // else, change directions and decrease CMPB
        //
        if(epwm_info->EPwm_CMPB_Direction == EPWM_CMP_UP)
        {
            if(epwm_info->EPwmRegHandle->CMPB.bit.CMPB < epwm_info->EPwmMaxCMPB)
            {
                epwm_info->EPwmRegHandle->CMPB.bit.CMPB++;
            }
            else
            {
                epwm_info->EPwm_CMPB_Direction = EPWM_CMP_DOWN;
                epwm_info->EPwmRegHandle->CMPB.bit.CMPB--;
            }
        }

        //
        // If we were decreasing CMPB, check to see if
        // we reached the min value.  If not, decrease CMPB
        // else, change directions and increase CMPB
        //
        else
        {
            if(epwm_info->EPwmRegHandle->CMPB.bit.CMPB == epwm_info->EPwmMinCMPB)
            {
                epwm_info->EPwm_CMPB_Direction = EPWM_CMP_UP;
                epwm_info->EPwmRegHandle->CMPB.bit.CMPB++;
            }
            else
            {
                epwm_info->EPwmRegHandle->CMPB.bit.CMPB--;
            }
        }
    }
    else
    {
        epwm_info->EPwmTimerIntCount++;
    }

    return;
}
